<?php
/*
Plugin Name: WooCommerce Buy For Me
Plugin URI: http://www.burnhambox.com/buy-for-me
Description: Allow your users to discover the enjoyable way of asking for a gift from a friend.
Version: 3.2.1
Author: پرشین اسکریپت
Author URI: http://www.persianscript
Text Domain: buyforme
Domain Path: /languages/
License: GNU General Public License version 3.0
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/

/*////////////////////////////////
////////// Localization //////////
////////////////////////////////*/

load_plugin_textdomain( 'buyforme', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

/*/// END - Localization ///*/

/*///////////////////////////////////////
////////// WC_Buy_For_Me Class //////////
///////////////////////////////////////*/

class WC_Buy_For_Me {

	/*/////////////////////////////
	////////// Construct //////////
	/////////////////////////////*/

	public function __construct() {

		/*////////////////////////////
		////////// Settings //////////
		////////////////////////////*/

		$this->logo_path_default = plugin_dir_url( __FILE__ ) . 'assets/images/sample-logo.png';
		$this->bfm_active = get_option( 'bfm_active', 'yes' ) == 'yes' ? true : false;
		$this->bfm_default_enabled = get_option( 'bfm_default_enabled', 'yes' ) == 'yes' ? true : false;
		$this->bfm_loop_enabled = get_option( 'bfm_loop_enabled', 'yes' ) == 'yes' ? true : false;
		$this->bfm_product_enabled = get_option( 'bfm_product_enabled', 'yes' ) == 'yes' ? true : false;
		$this->bfm_button_repeat = get_option( 'bfm_button_repeat', 'no' ) == 'yes' ? true : false;
		$this->bfm_disclaimer = get_option( 'bfm_disclaimer', 'no' ) == 'yes' ? true : false;
		$this->bfm_override_colors = get_option( 'bfm_override_colors', 'yes' ) == 'yes' ? true : false;
		$this->bfm_pot = get_option( 'bfm_pot', 'no' ) == 'yes' ? true : false;
		$this->bfm_button_container = get_option( 'bfm_button_container', 'no' ) == 'yes' ? true : false;
		$this->bfm_hidden_price = get_option( 'bfm_hidden_price', 'yes' ) == 'yes' ? true : false;
		$this->bfm_fa_enabled = get_option( 'bfm_fa_enabled', 'no' ) == 'yes' ? true : false;

		add_option( 'bfm_active', 'yes' );
		add_option( 'bfm_default_enabled', 'yes' );
		add_option( 'bfm_loop_enabled', 'yes' );
		add_option( 'bfm_product_enabled', 'yes' );
		add_option( 'bfm_button_repeat', 'no' );
		add_option( 'bfm_disclaimer', 'no' );
		add_option( 'bfm_override_colors', 'yes' );
		add_option( 'bfm_pot', 'no' );
		add_option( 'bfm_button_container', 'no' );
		add_option( 'bfm_hidden_price', 'yes' );
		add_option( 'bfm_fa_enabled', 'no' );

		$this->settings = array(

			// General Settings
			array(
				'name' => esc_html__( 'General Settings', 'buyforme' ),
				'id' => 'bfm_section_start_general',
				'type' => 'title',
			),

			array(
				'name' => esc_html__( 'Logo Image Path', 'buyforme' ),
				'desc' => esc_html__( 'Write the full path of the logo. Leave empty to display site title.', 'buyforme' ),
				'id' => 'bfm_logo_path2',
				'type' => 'text',
				'default' => esc_url( $this->logo_path_default ),
			),

			array(
				'name' => esc_html__( 'Logo Image Height', 'buyforme' ),
				'desc' => 'Min: 20 - Max: 60',
				'id' => 'bfm_logo_height',
				'type' => 'number',
				'default' => 60,
				'css' => 'width: 50px;',
			),

			array(
				'name' => esc_html__( 'Enable Buy For Me', 'buyforme' ),
				'id' => 'bfm_active',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Show \'Buy For Me\' button on Shop/Category pages.', 'buyforme' ),
				'id' => 'bfm_loop_enabled',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Show \'Buy For Me\' button on Product pages.', 'buyforme' ),
				'id' => 'bfm_product_enabled',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Place \'Buy For Me\' button below other elements on product pages.', 'buyforme' ),
				'id' => 'bfm_button_container',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Show \'Buy For Me\' button for new products by default.', 'buyforme' ),
				'id' => 'bfm_default_enabled',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Show \'Hide Price\' option.', 'buyforme' ),
				'id' => 'bfm_hidden_price',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Trigger \'Buy For Me\' action while additional WooCommerce plugins are active. (e.g. Quick View)', 'buyforme' ),
				'id' => 'bfm_button_repeat',
				'type' => 'checkbox',
			),

			array(
				'desc' => esc_html__( 'Show disclaimer notice at the bottom of the sent e-mail.', 'buyforme' ),
				'id' => 'bfm_disclaimer',
				'type' => 'checkbox',
			),

			array(
				'name' => esc_html__( 'Font Awesome Icon', 'buyforme' ),
				'desc' => __( 'e.g. fa-gift &nbsp;&nbsp;&nbsp; See <a href="http://fontawesome.io/cheatsheet/" target="_blank">Font Awesome Cheatsheet</a> for more icons.', 'buyforme' ),
				'id' => 'bfm_fa',
				'type' => 'text',
				'default' => '',
			),

			array(
				'desc' => esc_html__( 'Enable Font Awesome support.', 'buyforme' ),
				'id' => 'bfm_fa_enabled',
				'type' => 'checkbox',
			),

			array(
				'name' => esc_html__( 'Product Image Size', 'buyforme' ),
				'id' => 'bfm_product_image_size',
				'type' => 'radio',
				'options' => array(
					'shop_catalog' => esc_html__( 'Catalog Image', 'buyforme' ),
					'shop_single' => esc_html__( 'Single Product Image', 'buyforme' ),
					'shop_thumbnail' => esc_html__( 'Product Thumbnail', 'buyforme' ),
				),
				'default' => 'shop_thumbnail',
			),

			array(
				'name' => esc_html__( 'Tracking Parameters', 'buyforme' ),
				'desc' => esc_html__( 'e.g. ?utm_campaign=buyforme&utm_medium=email&utm_source=friend', 'buyforme' ),
				'id' => 'bfm_tracking_parameter',
				'type' => 'text',
				'default' => '?buyforme=true',
			),

			array(
				'name' => esc_html__( '"Bcc:" E-mail Address', 'buyforme' ),
				'desc' => esc_html__( 'When a visitor uses Buy For Me, you get a copy of the sent e-mail. Only you can see this e-mail address.', 'buyforme' ),
				'id' => 'bfm_bcc',
				'type' => 'text',
			),

			array(
				'name' => esc_html__( 'E-mail Subject Prefix', 'buyforme' ),
				'id' => 'bfm_email_subject_prefix',
				'type' => 'radio',
				'options' => array(
					'prefix_sender' => esc_html__( 'Add sender name to the sent e-mail subject. (e.g. John Doe says: ...)', 'buyforme' ),
					'prefix_receiver' => esc_html__( 'Add receiver name to the sent e-mail subject. (e.g. Hi Jane Doe, ...)', 'buyforme' ),
					'prefix_none' => esc_html__( 'None', 'buyforme' ),
				),
				'default' => 'prefix_none',
			),

			array(
				'id' => 'bfm_section_end_general',
				'type' => 'sectionend',
			),

			// Colors
			array(
				'name' => esc_html__( 'Color Options', 'buyforme' ),
				'id' => 'bfm_section_start_colors',
				'type' => 'title',
			),

			array(
				'name' => esc_html__( 'Use a Single Color Scheme', 'buyforme' ),
				'id' => 'bfm_single_scheme',
				'type' => 'select',
				'options' => array(
					'disabled' => esc_html__( 'Disabled', 'buyforme' ),
					'custom' => esc_html__( 'Custom', 'buyforme' ),
					'default' => 'Default',
					'lollipop' => 'Lollipop',
					'macaron' => 'Macaron',
					'blueberry' => 'Blueberry',
					'forest' => 'Forest',
					'dark' => 'Dark',
					'lemonade' => 'Lemonade',
					'sailor' => 'Sailor',
					'gold' => 'Gold',
				),
				'default' => 'disabled',
			),

			array(
				'id' => 'bfm_section_end_color_options',
				'type' => 'sectionend',
			),

			array(
				'name' => esc_html__( 'Custom Color Scheme', 'buyforme' ),
				'desc' => esc_html__( 'Only used if "Custom" is selected from the "Use a Single Color Scheme" box above.', 'buyforme' ),
				'id' => 'bfm_section_custom_colors',
				'type' => 'title',
			),

			array(
				'name' => esc_html__( 'Background Color', 'buyforme' ),
				'id' => 'bfm_bgColor',
				'type' => 'color',
				'default' => '#000000',
			),

			array(
				'name' => esc_html__( 'Foreground Color', 'buyforme' ),
				'id' => 'bfm_fgColor',
				'type' => 'color',
				'default' => '#FFFFFF',
			),

			array(
				'name' => esc_html__( 'Message Text Color', 'buyforme' ),
				'id' => 'bfm_messageTextColor',
				'type' => 'color',
				'default' => '#333333',
			),

			array(
				'name' => esc_html__( 'Product Text Color', 'buyforme' ),
				'id' => 'bfm_productTextColor',
				'type' => 'color',
				'default' => '#999999',
			),

			array(
				'name' => esc_html__( 'Button Text Color', 'buyforme' ),
				'id' => 'bfm_buttonTextColor',
				'type' => 'color',
				'default' => '#FFFFFF',
			),

			array(
				'name' => esc_html__( 'Button Background Color', 'buyforme' ),
				'id' => 'bfm_buttonBgColor',
				'type' => 'color',
				'default' => '#F00000',
			),

			array(
				'id' => 'bfm_section_end_custom_colors',
				'type' => 'sectionend',
			),

			array(
				'name' => esc_html__( 'Window Colors', 'buyforme' ),
				'id' => 'bfm_section_window_colors',
				'type' => 'title',
			),

			array(
				'name' => esc_html__( 'Window Background Color', 'buyforme' ),
				'id' => 'bfm_color_window_bg',
				'type' => 'color',
				'default' => '#edeae3',
			),

			array(
				'desc' => esc_html__( 'Override theme colors with the ones below.', 'buyforme' ),
				'id' => 'bfm_override_colors',
				'type' => 'checkbox',
			),

			array(
				'name' => esc_html__( 'Text Color', 'buyforme' ),
				'id' => 'bfm_color_text',
				'type' => 'color',
				'default' => '#666666',
			),

			array(
				'name' => esc_html__( 'Heading (h2) Color', 'buyforme' ),
				'id' => 'bfm_color_title',
				'type' => 'color',
				'default' => '#3c2313',
			),

			array(
				'name' => esc_html__( 'Input Background Color', 'buyforme' ),
				'id' => 'bfm_color_input_bg',
				'type' => 'color',
				'default' => '#ffffff',
			),

			array(
				'name' => esc_html__( 'Input Text Color', 'buyforme' ),
				'id' => 'bfm_color_input_text',
				'type' => 'color',
				'default' => '#666666',
			),

			array(
				'id' => 'bfm_section_end_colors',
				'type' => 'sectionend',
			),

			// Translation
			array(
				'name' => esc_html__( 'Translation', 'buyforme' ),
				'desc' => esc_html__( 'You can use the .pot file for your translations as usual. But, if you wish, you can disable it and use the fields below.', 'buyforme' ),
				'id' => 'bfm_section_start_translation',
				'type' => 'title',
			),

			array(
				'name' => esc_html__( 'Disable Language File', 'buyforme' ),
				'desc' => esc_html__( 'Disable plugin\'s language file (.pot) and use the translations below.', 'buyforme' ),
				'id' => 'bfm_pot',
				'type' => 'checkbox',
			),

			array(
				'name' => esc_html__( 'Button Text', 'buyforme' ),
				'desc' => esc_html__( 'Buy For Me', 'buyforme' ),
				'id' => 'bfm_tra_button_name',
				'type' => 'text',
				'default' => 'Buy For Me',
			),

			array(
				'name' => esc_html__( 'Window Strings', 'buyforme' ),
				'desc' => esc_html__( 'E-mail Preview', 'buyforme' ),
				'id' => 'bfm_tra_preview',
				'type' => 'text',
				'default' => 'E-mail Preview',
			),

			array(
				'desc' => esc_html__( 'Options & Settings', 'buyforme' ),
				'id' => 'bfm_tra_design',
				'type' => 'text',
				'default' => 'Options & Settings',
			),

			array(
				'desc' => esc_html__( 'Color Scheme', 'buyforme' ),
				'id' => 'bfm_tra_color',
				'type' => 'text',
				'default' => 'Color Scheme',
			),

			array(
				'desc' => esc_html__( 'Default', 'buyforme' ),
				'id' => 'bfm_tra_default',
				'type' => 'text',
				'default' => 'Default',
			),

			array(
				'desc' => esc_html__( 'Button Text', 'buyforme' ),
				'id' => 'bfm_tra_button',
				'type' => 'text',
				'default' => 'Button Text',
			),

			array(
				'desc' => esc_html__( 'Your Message', 'buyforme' ),
				'id' => 'bfm_tra_message',
				'type' => 'text',
				'default' => 'Your Message',
			),

			array(
				'desc' => esc_html__( 'Hide Price', 'buyforme' ),
				'id' => 'bfm_tra_price',
				'type' => 'text',
				'default' => 'Hide Price',
			),

			array(
				'desc' => esc_html__( 'Your Name', 'buyforme' ),
				'id' => 'bfm_tra_name',
				'type' => 'text',
				'default' => 'Your Name',
			),

			array(
				'desc' => esc_html__( 'Your E-mail Address', 'buyforme' ),
				'id' => 'bfm_tra_email',
				'type' => 'text',
				'default' => 'Your E-mail Address',
			),

			array(
				'desc' => esc_html__( 'Receiver Name', 'buyforme' ),
				'id' => 'bfm_tra_r_name',
				'type' => 'text',
				'default' => 'Receiver Name',
			),

			array(
				'desc' => esc_html__( 'Receiver E-mail Address', 'buyforme' ),
				'id' => 'bfm_tra_receiver',
				'type' => 'text',
				'default' => 'Receiver E-mail Address',
			),

			array(
				'desc' => esc_html__( 'E-mail Subject', 'buyforme' ),
				'id' => 'bfm_tra_subject',
				'type' => 'text',
				'default' => 'E-mail Subject',
			),

			array(
				'desc' => esc_html__( 'Send E-mail', 'buyforme' ),
				'id' => 'bfm_tra_send',
				'type' => 'text',
				'default' => 'Send E-mail',
			),

			array(
				'name' => esc_html__( 'Warning Messages', 'buyforme' ),
				'desc' => esc_html__( 'Your e-mail has been sent! You will also get a copy of it to your own e-mail address.', 'buyforme' ),
				'id' => 'bfm_tra_sent',
				'type' => 'text',
				'default' => 'Your e-mail has been sent! You will also get a copy of it to your own e-mail address.',
			),

			array(
				'desc' => esc_html__( 'A problem has occured. Please try again.', 'buyforme' ),
				'id' => 'bfm_tra_problem',
				'type' => 'text',
				'default' => 'A problem has occured. Please try again.',
			),

			array(
				'name' => esc_html__( 'Preview Defaults', 'buyforme' ),
				'desc' => esc_html__( 'Would you please buy this awesome piece for me dear? :)', 'buyforme' ),
				'id' => 'bfm_tra_p_Message',
				'type' => 'text',
				'default' => 'Would you please buy this awesome piece for me dear? :)',
			),

			array(
				'desc' => esc_html__( 'SHOP NOW', 'buyforme' ),
				'id' => 'bfm_tra_p_Button',
				'type' => 'text',
				'default' => 'SHOP NOW',
			),

			array(
				'desc' => esc_html__( 'I can forgive you if you buy this :)', 'buyforme' ),
				'id' => 'bfm_tra_p_Subject',
				'type' => 'text',
				'default' => 'I can forgive you if you buy this :)',
			),

			array(
				'name' => esc_html__( 'E-mail Subject Prefix Word for Sender Name', 'buyforme' ),
				'desc' => esc_html__( '(Sender Name) says', 'buyforme' ),
				'id' => 'bfm_tra_says',
				'type' => 'text',
				'default' => 'says',
			),

			array(
				'name' => esc_html__( 'E-mail Subject Prefix Word for Receiver Name', 'buyforme' ),
				'desc' => esc_html__( 'Hi (Receiver Name)', 'buyforme' ),
				'id' => 'bfm_tra_hi',
				'type' => 'text',
				'default' => 'Hi',
			),

			array(
				'name' => esc_html__( 'Disclaimer Notice', 'buyforme' ),
				'id' => 'bfm_tra_disclaimer',
				'type' => 'textarea',
				'css' => 'width: 100%; max-width: 400px; height: 100px;',
				'default' => 'This e-mail has been sent by %name% (%email%) via our online store.',
			),

			array(
				'id' => 'bfm_section_end_translation',
				'type' => 'sectionend',
			),

			// Custom CSS
			array(
				'name' => esc_html__( 'Custom CSS', 'buyforme' ),
				'desc' => esc_html__( 'You can override the plugin styles by placing your custom CSS code below.', 'buyforme' ),
				'id' => 'bfm_section_start_customCSS',
				'type' => 'title',
			),

			array(
				'name' => esc_html__( 'Your CSS Code', 'buyforme' ),
				'id' => 'bfm_custom_css',
				'type' => 'textarea',
				'css' => 'width: 100%; max-width: 400px; height: 200px;',
			),

			array(
				'id' => 'bfm_section_end_customCSS',
				'type' => 'sectionend',
			),

		);

		/*/// END - Settings ///*/

		/*/////////////////////////////////////
		////////// Actions & Filters //////////
		/////////////////////////////////////*/

		// Enqueue styles & scripts
		add_action( 'wp_enqueue_scripts', array( $this, 'buy_for_me_enqueue_styles_scripts' ) );

		// Attach Buy For Me Button
		if ( $this->bfm_loop_enabled ) {

			add_action( 'woocommerce_after_shop_loop_item', array( $this, 'buy_for_me_call_button' ), 20 );

		}

		if ( $this->bfm_product_enabled ) {

			add_action( 'woocommerce_after_add_to_cart_button', array( $this, 'buy_for_me_call_button' ), 10 );

		}

		// Write Panels
		add_action( 'woocommerce_product_options_pricing', array( $this, 'buy_for_me_write_panel' ) );
		add_action( 'woocommerce_process_product_meta', array( $this, 'buy_for_me_write_panel_save' ) );

		// Admin
		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'buy_for_me_add_settings_tab' ), 50 );
		add_action( 'woocommerce_settings_tabs_buy_for_me', array( $this, 'buy_for_me_settings_tab' ) );
		add_action( 'woocommerce_update_options_buy_for_me', array( $this, 'buy_for_me_update_settings' ) );

		// E-mail Content & Preview
		add_action( 'bfm_get_mail_content', array( $this, 'buy_for_me_getMailContentPreview' ) );
		add_action( 'bfm_check_mail', array( $this, 'buy_for_me_sendMail' ) );

		/*/// END - Actions & Filters ///*/

	}

	/*/// END - Construct ///*/

	/*////////////////////////////////////////////
	////////// Enqueue Styles & Scripts //////////
	////////////////////////////////////////////*/

	public function buy_for_me_enqueue_styles_scripts() {

		wp_enqueue_script( 'buyforme-js', plugin_dir_url( __FILE__ ) . 'assets/js/bfm.js', array( 'jquery' ), '1.0', true );
		wp_enqueue_style( 'buyforme-css', plugin_dir_url( __FILE__ ) . 'assets/css/bfm.css' );

		if ( $this->bfm_fa_enabled ) {

			wp_enqueue_style( 'font-awesome', plugin_dir_url( __FILE__ ) . 'assets/css/font-awesome.min.css', array( 'buyforme-css' ), '4.7.0', 'all' );

		}

		$logo_height = get_option( 'bfm_logo_height', 60 );
		if ( $logo_height < 20 ) { $logo_height = 20; } else if ( $logo_height > 60 ) { $logo_height = 60; }

		$buy_for_me_css = '

			#bfm-logo img { max-height: ' . esc_attr( $logo_height ) . 'px; }
			.bfm-container, .bfm-message { background-color: ' . get_option( 'bfm_color_window_bg', '#edeae3' ) . '; }

		';

		if ( $this->bfm_override_colors ) {

			$buy_for_me_css .= '

				#brnhmbx-buy-for-me { color: ' . get_option( 'bfm_color_text', '#666666' ) . ' !important; }
				.bfm-message, #brnhmbx-buy-for-me h2 { color: ' . get_option( 'bfm_color_title', '#3c2313' ) . ' !important; }
				.bfm-input, .bfm-select { border: none !important; background-color: ' . get_option( 'bfm_color_input_bg', '#ffffff' ) . ' !important; color: ' . get_option( 'bfm_color_input_text', '#666666' ) . ' !important; }

			';

		}

		$get_single_scheme = get_option( 'bfm_single_scheme', 'disabled' );
		if ( $get_single_scheme != 'disabled' ) { $buy_for_me_css .= 'label[for="bfm-color-scheme"], select#bfm-color-scheme { display: none !important; } .col-bfm-button-text, #bfm-button-text { max-width: 100%; }'; }
		$buy_for_me_css .= esc_html( get_option( 'bfm_custom_css', '' ) );

		wp_add_inline_style( 'buyforme-css', $buy_for_me_css );

	}

	/*/// END - Enqueue Styles & Scripts ///*/

	/*//////////////////////////////////////////
	////////// Load Buy For Me Window //////////
	//////////////////////////////////////////*/

	public function buy_for_me_call_window() {

		echo '<span id="bfm_button_repeat">' . esc_html( $this->bfm_button_repeat ) . '</span>';
		wc_get_template( 'bfm-window.php', array(), 'buyforme', plugin_dir_path( __FILE__ ) . '/templates/' );

	}

	/*/// END - Load Buy For Me Window ///*/

	/*//////////////////////////////////////////
	////////// Call Buy For Me Button //////////
	//////////////////////////////////////////*/

	public function buy_for_me_call_button() {

		global $post;
		global $product;

		$is_bfm = get_post_meta( $post->ID, '_is_bfm', true );

		if ( $is_bfm == '' && $this->bfm_default_enabled ) {

			$is_bfm = 'yes';

		}

		if ( $is_bfm == 'yes' && $this->bfm_active && $product->is_in_stock() && !$product->is_type( 'grouped' ) ) {

			wc_get_template( 'bfm-button.php', array(), 'buyforme', plugin_dir_path( __FILE__ ) . '/templates/' );

		}

	}

	/*/// END - Call Buy For Me Button ///*/

	/*///////////////////////////////////////////////////////////
	////////// Enable/Disable Buy For Me for a Product //////////
	///////////////////////////////////////////////////////////*/

	public function buy_for_me_write_panel() {

		global $post;

		echo '</div><div class="options_group show_if_simple show_if_variable show_if_external">';

		$is_bfm = get_post_meta( $post->ID, '_is_bfm', true );

		if ( $is_bfm == '' && $this->bfm_default_enabled ) {

			$is_bfm = 'yes';

		}

		woocommerce_wp_checkbox( array(

			'id'            => '_is_bfm',
			'wrapper_class' => '',
			'value'         => $is_bfm,
			'label'         => 'Buy For Me',
			'description'   => esc_html__( 'Show \'Buy For Me\' button for this product.', 'buyforme' ),

		) );

	}

	public function buy_for_me_write_panel_save( $post_id ) {

		$_is_bfm = ! empty( $_POST['_is_bfm'] ) ? 'yes' : 'no';
		update_post_meta( $post_id, '_is_bfm', $_is_bfm );

	}

	/*/// END - Enable/Disable Buy For Me for a Product ///*/

	/*//////////////////////////////////////
	////////// Admin Settings Tab //////////
	//////////////////////////////////////*/

	public function buy_for_me_add_settings_tab( $settings_tabs ) {

		$settings_tabs['buy_for_me'] = 'Buy For Me';
		return $settings_tabs;

	}

	public function buy_for_me_settings_tab() {

		woocommerce_admin_fields( $this->settings );

	}

	public function buy_for_me_update_settings() {

		woocommerce_update_options( $this->settings );

	}

	/*/// END - Admin Settings Tab ///*/

	/*////////////////////////////////
	////////// Form Actions //////////
	////////////////////////////////*/

	public function buy_for_me_sendMail() {

		if ( isset( $_POST[ 'bfm-submit' ] ) ) {

			$this->buy_for_me_buyForMe();

		}

	}

	public function buy_for_me_buyForMe() {

		// Translation
		if ( get_option( 'bfm_pot', 'no' ) == 'no' ) {

			// If .pot file enabled
			$bfm_tra_sent = esc_html__( 'Your e-mail has been sent! You will also get a copy of it to your own e-mail address.', 'buyforme' );
			$bfm_tra_problem = esc_html__( 'A problem has occured. Please try again.', 'buyforme' );
			$bfm_tra_hi = esc_html__( 'Hi', 'buyforme' );
			$bfm_tra_says = esc_html__( 'says', 'buyforme' );

		} else {

			// If .pot file disabled
			$bfm_tra_sent = get_option( 'bfm_tra_sent', 'Your e-mail has been sent! You will also get a copy of it to your own e-mail address.' );
			$bfm_tra_problem = get_option( 'bfm_tra_problem', 'A problem has occured. Please try again.' );
			$bfm_tra_hi = get_option( 'bfm_tra_hi', 'Hi' );
			$bfm_tra_says = get_option( 'bfm_tra_says', 'says' );

		}

		// Inputs

		$bfm_tra_prefix_final = '';

		if ( isset( $_POST[ 'bfm-receiver-name' ] ) ) {

			if ( get_option( 'bfm_email_subject_prefix', 'prefix_none' ) == 'prefix_receiver' ) {

				$bfm_tra_prefix_final = esc_html( $bfm_tra_hi ) . ' ' . esc_html( $_POST[ 'bfm-receiver-name' ] ) . ', ';

			}

		}

		if ( isset( $_POST[ 'bfm-receiver-email' ] ) ) {

			$get_toEmail = $_POST[ 'bfm-receiver-email' ];

		}

		if ( isset( $_POST[ 'bfm-your-name' ] ) ) {

			$get_fromName = $_POST[ 'bfm-your-name' ];

			if ( get_option( 'bfm_email_subject_prefix', 'prefix_none' ) == 'prefix_sender' ) {

				$bfm_tra_prefix_final = esc_html( $get_fromName ) . ' ' . esc_html( $bfm_tra_says ) . ': ';

			}

		}

		if ( isset( $_POST[ 'bfm-your-email' ] ) ) {

			$get_fromEmail = $_POST[ 'bfm-your-email' ];

		}

		if ( isset( $_POST[ 'bfm-email-subject' ] ) ) {

			$get_subject = $_POST[ 'bfm-email-subject' ];

		}

		$get_owner_bcc = get_option( 'bfm_bcc', '' );

		$to = $get_toEmail;
		$subject = esc_html( $bfm_tra_prefix_final ) . esc_html( $get_subject );
		$content = $this->buy_for_me_getMailContent();
		$headers[] = 'Content-Type: text/html';
		$headers[] = 'Reply-To: ' . esc_html( $get_fromName ) . ' <' . esc_html( $get_fromEmail ) . '>';
		$headers[] = 'Bcc: <' . esc_html( $get_fromEmail ) . '>';

		if ( $get_owner_bcc ) {

			$headers[] = 'Bcc: <' . esc_html( $get_owner_bcc ) . '>';

		}

		$bfm_status = wp_mail( $to, $subject, $content, $headers );

		if ( $bfm_status ) {

			$bfm_return = esc_html( $bfm_tra_sent );

		} else {

			$bfm_return = esc_html( $bfm_tra_problem );

		}

		if ( !is_admin() ) {

			// Result
			echo '<span id="bfm_return">' . esc_html( $bfm_return ) . '</span>';

		}

	}

	public function buy_for_me_getMailContent() {

		// Translation
		if ( get_option( 'bfm_pot', 'no' ) == 'no' ) {

			// If .pot file enabled
			$get_disclaimer = esc_html__( 'This e-mail has been sent by %name% (%email%) via our online store.', 'buyforme' );

		} else {

			// If .pot file disabled
			$get_disclaimer = wp_kses_post( get_option( 'bfm_tra_disclaimer', 'This e-mail has been sent by %name% (%email%) via our online store.' ) );

		}

		$get_messageText = '';
		$get_buttonText = '';

		if ( isset( $_POST[ 'bfm-your-message' ] ) ) {

			$get_messageText = $_POST[ 'bfm-your-message' ];

		}

		if ( isset( $_POST[ 'bfm-button-text' ] ) ) {

			$get_buttonText = $_POST[ 'bfm-button-text' ];

		}

		if ( isset( $_POST[ 'bfm-your-name' ] ) ) {

			$get_fromName = $_POST[ 'bfm-your-name' ];

		}

		if ( isset( $_POST[ 'bfm-your-email' ] ) ) {

			$get_fromEmail = $_POST[ 'bfm-your-email' ];

		}

		$product_ID = $_POST[ 'bfm-product-id' ];
		$product = new WC_Product( $product_ID );

		if ( has_post_thumbnail( $product_ID ) ) {

			$get_previewImgSize = get_option( 'bfm_product_image_size', 'shop_thumbnail' );
			$product_Image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_ID ), $get_previewImgSize );
			$get_productImg = $product_Image[0];

		} else {

			$get_productImg = '';

		}

		$get_productText = get_the_title( $product_ID );
		$get_priceText = $product->get_price_html();
		$get_productURL = esc_url( get_permalink( $product_ID ) );

		$get_siteURL = esc_url( site_url() );
		$get_tracking = get_option( 'bfm_tracking_parameter', '?buyforme=true' );

		// Color Schemes
		$get_single_scheme = get_option( 'bfm_single_scheme', 'disabled' );
		$bfm_color_schemes = array(

			array( 'name' => 'default', 'bg' => '#76b9bb', 'fg' => '#FFFFFF', 'message' => '#999999', 'text' => '#3c2313', 'button_text' => '#3c2313', 'button_bg' => '#dedacc' ),
			array( 'name' => 'lollipop', 'bg' => '#dba5c6', 'fg' => '#FFFFFF', 'message' => '#999999', 'text' => '#bd448e', 'button_text' => '#FFFFFF', 'button_bg' => '#c2c2c2' ),
			array( 'name' => 'macaron', 'bg' => '#f9b9ad', 'fg' => '#FFFFFF', 'message' => '#a26256', 'text' => '#999999', 'button_text' => '#FFFFFF', 'button_bg' => '#b64646' ),
			array( 'name' => 'blueberry', 'bg' => '#8f8ead', 'fg' => '#FFFFFF', 'message' => '#262d70', 'text' => '#6a556d', 'button_text' => '#FFFFFF', 'button_bg' => '#a97baf' ),
			array( 'name' => 'forest', 'bg' => '#003333', 'fg' => '#81aaaa', 'message' => '#FFFFFF', 'text' => '#FFFFFF', 'button_text' => '#716f36', 'button_bg' => '#d2d1b9' ),
			array( 'name' => 'dark', 'bg' => '#999999', 'fg' => '#333333', 'message' => '#FFFFFF', 'text' => '#c7c7c7', 'button_text' => '#999999', 'button_bg' => '#1c1c1c' ),
			array( 'name' => 'lemonade', 'bg' => '#f5e4b2', 'fg' => '#FFFFFF', 'message' => '#b89b45', 'text' => '#b89b45', 'button_text' => '#65924a', 'button_bg' => '#c4eaad' ),
			array( 'name' => 'sailor', 'bg' => '#9c0e0e', 'fg' => '#FFFFFF', 'message' => '#555555', 'text' => '#777777', 'button_text' => '#FFFFFF', 'button_bg' => '#212d64' ),
			array( 'name' => 'gold', 'bg' => '#ac8051', 'fg' => '#FFFFFF', 'message' => '#caa976', 'text' => '#925e26', 'button_text' => '#FFFFFF', 'button_bg' => '#3c2313' ),

		);

		if ( $get_single_scheme == 'custom' ) {

			// Custom Color Scheme

			$get_bgColor = get_option( 'bfm_bgColor', '#000000' );
			$get_fgColor = get_option( 'bfm_fgColor', '#FFFFFF' );
			$get_messageTextColor = get_option( 'bfm_messageTextColor', '#333333' );
			$get_productTextColor = get_option( 'bfm_productTextColor', '#999999' );
			$get_buttonTextColor = get_option( 'bfm_buttonTextColor', '#FFFFFF' );
			$get_buttonBgColor = get_option( 'bfm_buttonBgColor', '#F00000' );

		} else {

			if ( $get_single_scheme == 'disabled' ) {

				$currentScheme = $_POST[ 'bfm-color-scheme' ];

			} else {

				$currentScheme = $get_single_scheme;

			}

			for ( $i = 0; $i < count( $bfm_color_schemes ); $i++ ) {

				if ( $bfm_color_schemes[$i][ 'name' ] == $currentScheme ) {

					$get_bgColor = $bfm_color_schemes[$i]['bg'];
					$get_fgColor = $bfm_color_schemes[$i]['fg'];
					$get_messageTextColor = $bfm_color_schemes[$i]['message'];
					$get_productTextColor = $bfm_color_schemes[$i]['text'];
					$get_buttonTextColor = $bfm_color_schemes[$i]['button_text'];
					$get_buttonBgColor = $bfm_color_schemes[$i]['button_bg'];
					break;

				}

			}

		}

		// E-mail Content
		$content = '<table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="' . esc_attr( $get_bgColor ) . '" style="background-color: ' . esc_attr( $get_bgColor ) . ';"><tr><td style="padding-top: 30px; padding-bottom: 30px;">';
		$content .= '<table border="0" cellpadding="0" cellspacing="0" align="center" width="300" bgcolor="' . esc_attr( $get_fgColor ) . '" style="background-color: ' . esc_attr( $get_fgColor ) . '; font-family: Arial, Helvetica, sans-serif; margin: 0 auto;">';
		$content .= '<tr><td style="text-align: center; padding-bottom: 20px; padding-top: 30px;">';
		$content .= '<a href="' . esc_url( $get_siteURL . $get_tracking ) . '" target="_blank" style="text-decoration: none; color: #3c2313; font-weight: bold;	font-size: 22px;">';

		if ( get_option( 'bfm_logo_path2', $this->logo_path_default ) ) {

			$logo_height = get_option( 'bfm_logo_height', 60 );
			if ( $logo_height < 20 ) { $logo_height = 20; } else if ( $logo_height > 60 ) { $logo_height = 60; }

			$content .= '<img src="' . esc_url( get_option( 'bfm_logo_path2', $this->logo_path_default ) ) . '" height="' . esc_attr( $logo_height ) . '" border="0" />';

		} else {

			$content .= '<span>' . esc_html( get_bloginfo( 'name' ) ) . '</span>';

		}

		$content .= '</a>';
		$content .= '</td></tr>';

		if ( $get_messageText ) {

			$content .= '<tr><td style="text-align: center; font-size: 16px; font-weight: bold; line-height: 22px; color: ' . esc_attr( $get_messageTextColor ) . '; padding-bottom: 20px; padding-left: 20px; padding-right: 20px;">';
			$content .= $get_messageText;
			$content .= '</td></tr>';

		}

		$content .= '<tr><td style="text-align: center; padding-bottom: 10px;">';
		$content .= '<a href="' . esc_url( $get_productURL . $get_tracking ) . '" target="_blank"><img src="' . esc_url( $get_productImg ) . '" width="180" border="0" /></a>';
		$content .= '</td></tr>';
		$content .= '<tr><td style="text-align: center; font-size: 14px; font-weight: bold; line-height: 18px; color: ' . esc_attr( $get_productTextColor ) . '; padding-bottom: 20px; padding-left: 20px; padding-right: 20px;">';
		$content .= esc_html( $get_productText );

		if ( !isset( $_POST[ 'bfm-hide-price' ] ) && $this->bfm_hidden_price ) {

			$content .= '<br /><span style="font-size: 12px; font-weight: bold;">' . wp_kses_post( $get_priceText ) . '</span>';

		}

		$content .= '</td></tr>';

		if ( $get_buttonText ) {

			$content .= '<tr><td style="text-align: center; padding-bottom: 30px;" height="50" width="180">';
			$content .= '<table border="0" cellpadding="0" cellspacing="0" align="center" width="180" height="50" style="margin: 0 auto;"><tr><td height="50" bgcolor="' . esc_attr( $get_buttonBgColor ) . '" style="background-color: ' . esc_attr( $get_buttonBgColor ) . '; text-align: center;"><a href="' . esc_url( $get_productURL . $get_tracking ) . '" target="_blank" style="font-weight: bold; font-family: Arial, Helvetica, sans-serif; font-size: 14px; color: ' . esc_attr( $get_buttonTextColor ) . '; text-decoration: none;">' . esc_html( $get_buttonText ) . '</a></td></tr></table>';
			$content .= '</td></tr>';

		}

		$content .= '</table>';
		$content .= '</td></tr>';

		if ( $this->bfm_disclaimer ) {

			$get_disclaimer_final = str_replace( array( '%name%', '%email%' ), array( $get_fromName, $get_fromEmail ), $get_disclaimer );

			$content .= '<tr><td style="padding-bottom: 30px; padding-left: 20px; padding-right: 20px;">';
			$content .= '<table style="max-width: 300px; margin: 0 auto; text-align: center; font-family: Arial, Helvetica, sans-serif; text-align: center; font-size: 12px; line-height: 16px; color: ' . esc_attr( $get_fgColor ) . ';"><tr><td>' . wp_kses_post( $get_disclaimer_final ) . '</td></tr></table>';
			$content .= '</td></tr>';

		}

		$content .= '</table>';

		return $content;

	}

	public function buy_for_me_getMailContentPreview() {

		// Translation
		if ( get_option( 'bfm_pot', 'no' ) == 'no' ) {

			// If .pot file enabled
			$get_messageText = esc_html__( 'Would you please buy this awesome piece for me dear? :)', 'buyforme' );
			$get_buttonText = esc_html__( 'SHOP NOW', 'buyforme' );

		} else {

			// If .pot file disabled
			$get_messageText = esc_html( get_option( 'bfm_tra_p_Message', 'Would you please buy this awesome piece for me dear? :)' ) );
			$get_buttonText = esc_html( get_option( 'bfm_tra_p_Button', 'SHOP NOW' ) );

		}

		$get_productText = '';
		$get_priceText = '';
		$get_productImg = '#';
		$get_productURL = '#';

		$get_siteURL = esc_url( site_url() );

		// Color Schemes
		$get_single_scheme = get_option( 'bfm_single_scheme', 'disabled' );
		$bfm_color_schemes = array(

			array( 'name' => 'default', 'bg' => '#76b9bb', 'fg' => '#FFFFFF', 'message' => '#999999', 'text' => '#3c2313', 'button_text' => '#3c2313', 'button_bg' => '#dedacc' ),
			array( 'name' => 'lollipop', 'bg' => '#dba5c6', 'fg' => '#FFFFFF', 'message' => '#999999', 'text' => '#bd448e', 'button_text' => '#FFFFFF', 'button_bg' => '#c2c2c2' ),
			array( 'name' => 'macaron', 'bg' => '#f9b9ad', 'fg' => '#FFFFFF', 'message' => '#a26256', 'text' => '#999999', 'button_text' => '#FFFFFF', 'button_bg' => '#b64646' ),
			array( 'name' => 'blueberry', 'bg' => '#8f8ead', 'fg' => '#FFFFFF', 'message' => '#262d70', 'text' => '#6a556d', 'button_text' => '#FFFFFF', 'button_bg' => '#a97baf' ),
			array( 'name' => 'forest', 'bg' => '#003333', 'fg' => '#81aaaa', 'message' => '#FFFFFF', 'text' => '#FFFFFF', 'button_text' => '#716f36', 'button_bg' => '#d2d1b9' ),
			array( 'name' => 'dark', 'bg' => '#999999', 'fg' => '#333333', 'message' => '#FFFFFF', 'text' => '#c7c7c7', 'button_text' => '#999999', 'button_bg' => '#1c1c1c' ),
			array( 'name' => 'lemonade', 'bg' => '#f5e4b2', 'fg' => '#FFFFFF', 'message' => '#b89b45', 'text' => '#b89b45', 'button_text' => '#65924a', 'button_bg' => '#c4eaad' ),
			array( 'name' => 'sailor', 'bg' => '#9c0e0e', 'fg' => '#FFFFFF', 'message' => '#555555', 'text' => '#777777', 'button_text' => '#FFFFFF', 'button_bg' => '#212d64' ),
			array( 'name' => 'gold', 'bg' => '#ac8051', 'fg' => '#FFFFFF', 'message' => '#caa976', 'text' => '#925e26', 'button_text' => '#FFFFFF', 'button_bg' => '#3c2313' ),

		);

		if ( $get_single_scheme == 'custom' ) {

			// Custom Color Scheme

			$get_bgColor = get_option( 'bfm_bgColor', '#000000' );
			$get_fgColor = get_option( 'bfm_fgColor', '#FFFFFF' );
			$get_messageTextColor = get_option( 'bfm_messageTextColor', '#333333' );
			$get_productTextColor = get_option( 'bfm_productTextColor', '#999999' );
			$get_buttonTextColor = get_option( 'bfm_buttonTextColor', '#FFFFFF' );
			$get_buttonBgColor = get_option( 'bfm_buttonBgColor', '#F00000' );

		} else if ( $get_single_scheme == 'disabled' ) {

			$get_bgColor = '#76b9bb';
			$get_fgColor = '#FFFFFF';
			$get_messageTextColor = '#999999';
			$get_productTextColor = '#3c2313';
			$get_buttonTextColor = '#3c2313';
			$get_buttonBgColor = '#dedacc';

		} else {

			for ( $i = 0; $i < count( $bfm_color_schemes ); $i++ ) {

				if ( $bfm_color_schemes[$i][ 'name' ] == $get_single_scheme ) {

					$get_bgColor = $bfm_color_schemes[$i]['bg'];
					$get_fgColor = $bfm_color_schemes[$i]['fg'];
					$get_messageTextColor = $bfm_color_schemes[$i]['message'];
					$get_productTextColor = $bfm_color_schemes[$i]['text'];
					$get_buttonTextColor = $bfm_color_schemes[$i]['button_text'];
					$get_buttonBgColor = $bfm_color_schemes[$i]['button_bg'];
					break;

				}

			}

		}

		// E-mail Preview
		$content_preview = '<div id="bfm-bg-front" style="background-color: ' . esc_attr( $get_bgColor ) . ';">';
		$content_preview .= '<div id="bfm-fg-front" style="background-color: ' . esc_attr( $get_fgColor ) . ';">';
		$content_preview .= '<div id="bfm-logo"><a href="' . esc_url( $get_siteURL ) . '" target="_blank">';

		if ( get_option( 'bfm_logo_path2', $this->logo_path_default ) ) {

			$content_preview .= '<img alt="" src="' . esc_url( get_option( 'bfm_logo_path2', $this->logo_path_default ) ) . '" />';

		} else {

			$content_preview .= '<span>' . esc_html( get_bloginfo( 'name' ) ) . '</span>';

		}

		$content_preview .= '</a></div>';
		$content_preview .= '<div id="bfm-your-message-front" style="color: ' . esc_attr( $get_messageTextColor ) . ';">' . esc_html( $get_messageText ) . '</div>';
		$content_preview .= '<div id="bfm-product-url-front"><a href="' . esc_url( $get_productURL ) . '" target="_blank"><img alt="" src="' . esc_url( $get_productImg ) . '" /></a></div>';
		$content_preview .= '<div id="bfm-product-text-front" style="color: ' . esc_attr( $get_productTextColor ) . ';"><span id="bfm-title-text-front">' . esc_html( $get_productText ) . '</span>';
		if ( get_option( 'bfm_hidden_price', 'yes' ) == 'yes' ) { $content_preview .= '<br /><span id="bfm-price-text-front">' . wp_kses_post( $get_priceText ) . '</span>'; }
		$content_preview .= '</div>';
		$content_preview .= '<div id="bfm-button-text-front"><div id="bfm-button-text-front-bg" style="background-color: ' . esc_attr( $get_buttonBgColor ) . ';"><span id="bfm-button-text-front-a" style="color: ' . esc_attr( $get_buttonTextColor ) . ';">' . esc_html( $get_buttonText ) . '</span></div></div>';
		$content_preview .= '</div></div>';

		echo wp_kses_post( $content_preview );

	}

	/*/// END - Form Actions ///*/

}

/*/// END - WC_Buy_For_Me Class ///*/

/*//////////////////////////////////////
////////// "From Name" Filter //////////
//////////////////////////////////////*/

function buy_for_me_from_name_filter( $name ) {

	if ( isset( $_POST[ 'bfm-your-name' ] ) ) {

		$get_fromName = $_POST[ 'bfm-your-name' ];

	}

  	return $get_fromName;

}

add_filter( 'wp_mail_from_name', 'buy_for_me_from_name_filter' );

/*/// END - "From Name" Filter ///*/

/*/////////////////////////////////////
////////// Launch Buy For Me //////////
/////////////////////////////////////*/

$class = new WC_Buy_For_Me();

/* */

/*/////////////////////////////
////////// Shortcode //////////
/////////////////////////////*/

function buy_for_me_shortcode() {

	global $post;
	global $product;
	global $class;

	$is_bfm = get_post_meta( $post->ID, '_is_bfm', true );

	if ( $is_bfm == '' && $class->bfm_default_enabled ) {

		$is_bfm = 'yes';

	}

	if ( $is_bfm == 'yes' && $class->bfm_active && $product->is_in_stock() && !$product->is_type( 'grouped' ) ) {

		wc_get_template( 'bfm-button.php', array(), 'buyforme', plugin_dir_path( __FILE__ ) . '/templates/' );

	}

}
add_shortcode( 'buyforme', 'buy_for_me_shortcode' );
/*/// END - Shortcode ///*/
